package javawrapper;

/**
 * Manages snag dynamics data and behaviors.
 * <p>Copyright: Copyright (c) 2007 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 *
 * <br>Edit history:
 * <br>------------------
 * <br>March 8, 2007: Created (LEM)
 */

public class SnagDynamicsBehaviors
    extends BehaviorTypeBase {

  /** Snag Decay Class Dynamics Tree Fall Beta Parameter, species-specific */
  protected ModelVector mp_fSnagDecompTreeFallBeta = new ModelVector(
      "Snag Decay Class Dynamics Tree Fall Beta",
      "sd_snagDecompTreefallBeta", "sd_sdtbVal", 0,
      ModelVector.FLOAT);

  /** Snag Decay Class Dynamics Snag Fall Beta Parameter, species-specific */
  protected ModelVector mp_fSnagDecompSnagFallBeta = new ModelVector(
      "Snag Decay Class Dynamics Snag Fall Beta",
      "sd_snagDecompSnagfallBeta", "sd_sdsbVal", 0, ModelVector.FLOAT);

  /** Snag Decay Class Dynamics Tree Fall Alpha Parameter */
  protected ModelFloat m_fSnagDecompTreeFallAlpha = new ModelFloat(0,
      "Snag Decay Class Dynamics Tree Fall Alpha",
      "sd_snagDecompTreefallAlpha");

  /** Snag Decay Class Dynamics Tree Fall Delta Parameter */
  protected ModelFloat m_fSnagDecompTreeFallDelta = new ModelFloat(0,
      "Snag Decay Class Dynamics Tree Fall Delta",
      "sd_snagDecompTreefallDelta");

  /** Snag Decay Class Dynamics Tree Fall Theta Parameter */
  protected ModelFloat m_fSnagDecompTreeFallTheta = new ModelFloat(0,
      "Snag Decay Class Dynamics Tree Fall Theta",
      "sd_snagDecompTreefallTheta");

  /** Snag Decay Class Dynamics Tree Fall Iota Parameter */
  protected ModelFloat m_fSnagDecompTreeFallIota = new ModelFloat(0,
      "Snag Decay Class Dynamics Tree Fall Iota",
      "sd_snagDecompTreefallIota");

  /** Snag Decay Class Dynamics Tree Fall Lambda Parameter */
  protected ModelFloat m_fSnagDecompTreeFallLambda = new ModelFloat(0,
      "Snag Decay Class Dynamics Tree Fall Lambda",
      "sd_snagDecompTreefallLambda");

  /** Snag Decay Class Dynamics Snag Fall Alpha Parameter */
  protected ModelFloat m_fSnagDecompSnagFallAlpha = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Alpha",
      "sd_snagDecompSnagfallAlpha");

  /** Snag Decay Class Dynamics Snag Fall Gamma 2 Parameter */
  protected ModelFloat m_fSnagDecompSnagFallGamma2 = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Gamma 2",
      "sd_snagDecompSnagfallGamma2");

  /** Snag Decay Class Dynamics Snag Fall Gamma 3 Parameter */
  protected ModelFloat m_fSnagDecompSnagFallGamma3 = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Gamma 3",
      "sd_snagDecompSnagfallGamma3");

  /** Snag Decay Class Dynamics Snag Fall Gamma 4 Parameter */
  protected ModelFloat m_fSnagDecompSnagFallGamma4 = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Gamma 4",
      "sd_snagDecompSnagfallGamma4");

  /** Snag Decay Class Dynamics Snag Fall Gamma 5 Parameter */
  protected ModelFloat m_fSnagDecompSnagFallGamma5 = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Gamma 5",
      "sd_snagDecompSnagfallGamma5");

  /** Snag Decay Class Dynamics Snag Fall Zeta Parameter */
  protected ModelFloat m_fSnagDecompSnagFallZeta = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Zeta",
      "sd_snagDecompSnagfallZeta");

  /** Snag Decay Class Dynamics Snag Fall Eta Parameter */
  protected ModelFloat m_fSnagDecompSnagFallEta = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Eta",
      "sd_snagDecompSnagfallEta");

  /** Snag Decay Class Dynamics Snag Fall Kappa Parameter */
  protected ModelFloat m_fSnagDecompSnagFallKappa = new ModelFloat(0,
      "Snag Decay Class Dynamics Snag Fall Kappa",
      "sd_snagDecompSnagfallKappa");

  /** Snag Decay Class Dynamics Live To One Transition Probability */
  protected ModelFloat m_fSnagDecompLiveTo1Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Live To Class 1 Prob (0-1)",
      "sd_snagDecompLiveTo1Prob");

  /** Snag Decay Class Dynamics Live To Two Transition Probability */
  protected ModelFloat m_fSnagDecompLiveTo2Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Live To Class 2 Prob (0-1)",
      "sd_snagDecompLiveTo2Prob");

  /** Snag Decay Class Dynamics Live To Three Transition Probability */
  protected ModelFloat m_fSnagDecompLiveTo3Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Live To Class 3 Prob (0-1)",
      "sd_snagDecompLiveTo3Prob");

  /** Snag Decay Class Dynamics Live To Four Transition Probability */
  protected ModelFloat m_fSnagDecompLiveTo4Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Live To Class 4 Prob (0-1)",
      "sd_snagDecompLiveTo4Prob");

  /** Snag Decay Class Dynamics Live To Five Transition Probability */
  protected ModelFloat m_fSnagDecompLiveTo5Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Live To Class 5 Prob (0-1)",
      "sd_snagDecompLiveTo5Prob");

  /** Snag Decay Class Dynamics One To One Transition Probability */
  protected ModelFloat m_fSnagDecomp1To1Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 1 To Class 1 Prob (0-1)",
      "sd_snagDecomp1To1Prob");

  /** Snag Decay Class Dynamics One To Two Transition Probability */
  protected ModelFloat m_fSnagDecomp1To2Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 1 To Class 2 Prob (0-1)",
      "sd_snagDecomp1To2Prob");

  /** Snag Decay Class Dynamics One To Three Transition Probability */
  protected ModelFloat m_fSnagDecomp1To3Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 1 To Class 3 Prob (0-1)",
      "sd_snagDecomp1To3Prob");

  /** Snag Decay Class Dynamics One To Four Transition Probability */
  protected ModelFloat m_fSnagDecomp1To4Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 1 To Class 4 Prob (0-1)",
      "sd_snagDecomp1To4Prob");

  /** Snag Decay Class Dynamics One To Five Transition Probability */
  protected ModelFloat m_fSnagDecomp1To5Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 1 To Class 5 Prob (0-1)",
      "sd_snagDecomp1To5Prob");

  /** Snag Decay Class Dynamics Two To Two Transition Probability */
  protected ModelFloat m_fSnagDecomp2To2Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 2 To Class 2 Prob (0-1)",
      "sd_snagDecomp2To2Prob");

  /** Snag Decay Class Dynamics Two To Three Transition Probability */
  protected ModelFloat m_fSnagDecomp2To3Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 2 To Class 3 Prob (0-1)",
      "sd_snagDecomp2To3Prob");

  /** Snag Decay Class Dynamics Two To Four Transition Probability */
  protected ModelFloat m_fSnagDecomp2To4Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 2 To Class 4 Prob (0-1)",
      "sd_snagDecomp2To4Prob");

  /** Snag Decay Class Dynamics Two To Five Transition Probability */
  protected ModelFloat m_fSnagDecomp2To5Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 2 To Class 5 Prob (0-1)",
      "sd_snagDecomp2To5Prob");

  /** Snag Decay Class Dynamics Three To Three Transition Probability */
  protected ModelFloat m_fSnagDecomp3To3Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 3 To Class 3 Prob (0-1)",
      "sd_snagDecomp3To3Prob");

  /** Snag Decay Class Dynamics Three To Four Transition Probability */
  protected ModelFloat m_fSnagDecomp3To4Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 3 To Class 4 Prob (0-1)",
      "sd_snagDecomp3To4Prob");

  /** Snag Decay Class Dynamics Three To Five Transition Probability */
  protected ModelFloat m_fSnagDecomp3To5Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 3 To Class 5 Prob (0-1)",
      "sd_snagDecomp3To5Prob");

  /** Snag Decay Class Dynamics Four To Four Transition Probability */
  protected ModelFloat m_fSnagDecomp4To4Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 4 To Class 4 Prob (0-1)",
      "sd_snagDecomp4To4Prob");

  /** Snag Decay Class Dynamics Four To Five Transition Probability */
  protected ModelFloat m_fSnagDecomp4To5Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 4 To Class 5 Prob (0-1)",
      "sd_snagDecomp4To5Prob");

  /** Snag Decay Class Dynamics Five To Five Transition Probability */
  protected ModelFloat m_fSnagDecomp5To5Prob = new ModelFloat(0,
      "Snag Decay Class Dynamics Class 5 To Class 5 Prob (0-1)",
      "sd_snagDecomp5To5Prob");
  
  /** Snag Decay Class Dynamics minimum snag break height */
  protected ModelFloat m_fSnagDecompMinSnagBreakHeight = new ModelFloat(0,
      "Snag Decay Class Dynamics Minimum Snag Break Height",
      "sd_minSnagBreakHeight");
  
  /** Snag Decay Class Dynamics maximum snag break height */
  protected ModelFloat m_fSnagDecompMaxSnagBreakHeight = new ModelFloat(0,
      "Snag Decay Class Dynamics Maximum Snag Break Height",
      "sd_maxSnagBreakHeight");
 
  /**
   * Constructor
   * @param oManager GUIManager object.
   * @throws ModelException won't.
   */
  public SnagDynamicsBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Snag dynamics", "snagDynamics");

    m_sHelpID = "data.snag_dynamics_behaviors";

    //Set up our child behavior vector
    mp_oChildBehaviors = new Behavior[1];
    mp_oChildBehaviors[0] = new Behavior("Snag Decay Class Dynamics",
                                         "Snag Decay Class Dynamics",
                                         "snag decay class dynamics",
                                         (float) 2.0);
    mp_oChildBehaviors[0].mp_oNewTreeDataMembers.add(new DataMember(
        "Snag Decay Class", "SnagDecayClass", DataMember.INTEGER));
    mp_oChildBehaviors[0].mp_oNewTreeDataMembers.add(new DataMember(
        "New Break Height", "NewBreakHeight", DataMember.FLOAT));
    mp_oChildBehaviors[0].mp_oNewTreeDataMembers.add(new DataMember(
        "Snag Old Break Height", "SnagOldBreakHeight", DataMember.FLOAT));
    mp_oChildBehaviors[0].mp_oNewTreeDataMembers.add(new DataMember(
        "Fall", "Fall", DataMember.BOOLEAN));
    //Substrate can only apply to snags and adults
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.ADULT, true);
    mp_oChildBehaviors[0].SetCanApplyTo(TreePopulation.SNAG, true);

    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompTreeFallAlpha);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagDecompTreeFallBeta);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompTreeFallDelta);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompTreeFallTheta);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompTreeFallIota);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompTreeFallLambda);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallAlpha);
    mp_oChildBehaviors[0].AddRequiredData(mp_fSnagDecompSnagFallBeta);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallGamma2);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallGamma3);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallGamma4);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallGamma5);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallZeta);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallEta);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompSnagFallKappa);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompLiveTo1Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompLiveTo2Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompLiveTo3Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompLiveTo4Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompLiveTo5Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp1To1Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp1To2Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp1To3Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp1To4Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp1To5Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp2To2Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp2To3Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp2To4Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp2To5Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp3To3Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp3To4Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp3To5Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp4To4Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp4To5Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp5To5Prob);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompMinSnagBreakHeight);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecompMaxSnagBreakHeight);
    mp_oChildBehaviors[0].AddRequiredData(m_fSnagDecomp5To5Prob);    

    mp_oAllData.add(m_fSnagDecompTreeFallAlpha);
    mp_oAllData.add(mp_fSnagDecompTreeFallBeta);
    mp_oAllData.add(m_fSnagDecompTreeFallDelta);
    mp_oAllData.add(m_fSnagDecompTreeFallTheta);
    mp_oAllData.add(m_fSnagDecompTreeFallIota);
    mp_oAllData.add(m_fSnagDecompTreeFallLambda);
    mp_oAllData.add(m_fSnagDecompSnagFallAlpha);
    mp_oAllData.add(mp_fSnagDecompSnagFallBeta);
    mp_oAllData.add(m_fSnagDecompSnagFallGamma2);
    mp_oAllData.add(m_fSnagDecompSnagFallGamma3);
    mp_oAllData.add(m_fSnagDecompSnagFallGamma4);
    mp_oAllData.add(m_fSnagDecompSnagFallGamma5);
    mp_oAllData.add(m_fSnagDecompSnagFallZeta);
    mp_oAllData.add(m_fSnagDecompSnagFallEta);
    mp_oAllData.add(m_fSnagDecompSnagFallKappa);
    mp_oAllData.add(m_fSnagDecompLiveTo1Prob);
    mp_oAllData.add(m_fSnagDecompLiveTo2Prob);
    mp_oAllData.add(m_fSnagDecompLiveTo3Prob);
    mp_oAllData.add(m_fSnagDecompLiveTo4Prob);
    mp_oAllData.add(m_fSnagDecompLiveTo5Prob);
    mp_oAllData.add(m_fSnagDecomp1To1Prob);
    mp_oAllData.add(m_fSnagDecomp1To2Prob);
    mp_oAllData.add(m_fSnagDecomp1To3Prob);
    mp_oAllData.add(m_fSnagDecomp1To4Prob);
    mp_oAllData.add(m_fSnagDecomp1To5Prob);
    mp_oAllData.add(m_fSnagDecomp2To2Prob);
    mp_oAllData.add(m_fSnagDecomp2To3Prob);
    mp_oAllData.add(m_fSnagDecomp2To4Prob);
    mp_oAllData.add(m_fSnagDecomp2To5Prob);
    mp_oAllData.add(m_fSnagDecomp3To3Prob);
    mp_oAllData.add(m_fSnagDecomp3To4Prob);
    mp_oAllData.add(m_fSnagDecomp3To5Prob);
    mp_oAllData.add(m_fSnagDecomp4To4Prob);
    mp_oAllData.add(m_fSnagDecomp4To5Prob);
    mp_oAllData.add(m_fSnagDecomp5To5Prob);
    mp_oAllData.add(m_fSnagDecompMinSnagBreakHeight);
    mp_oAllData.add(m_fSnagDecompMaxSnagBreakHeight);

    //Set up the grid
    DataMember[] p_oDataMembers = new DataMember[2];
    Grid oNewGrid;

    String sGridName = "Snag Decay Class Dynamics Basal Area";
    p_oDataMembers[0] = new DataMember("Live BA Per Ha",  "Live BA Per Ha",
                                         DataMember.FLOAT);
    p_oDataMembers[1] = new DataMember("Cut BA Per Ha",  "Cut BA Per Ha",
                                         DataMember.FLOAT);
    //Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("Snag Decay Class Dynamics").AddGrid(oNewGrid);

  }

  /**
   * Does nothing.
   * @param oPop Tree population
   * @throws ModelException Not thrown.
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {}

  /**
   * Validates the data in preparation for parameter file writing or some such.
   * @throws ModelException if any of the probabilities are not proportions,
   * or if the probabilities for a class don't add up to 1.
   * @param oPop Not used.
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {

    if (GetBehaviorByKey("Snag Decay Class Dynamics").m_bIsEnabled) {

      MakeSureIsProportion(m_fSnagDecompLiveTo1Prob);
      MakeSureIsProportion(m_fSnagDecompLiveTo2Prob);
      MakeSureIsProportion(m_fSnagDecompLiveTo3Prob);
      MakeSureIsProportion(m_fSnagDecompLiveTo4Prob);
      MakeSureIsProportion(m_fSnagDecompLiveTo5Prob);
      MakeSureIsProportion(m_fSnagDecomp1To1Prob);
      MakeSureIsProportion(m_fSnagDecomp1To2Prob);
      MakeSureIsProportion(m_fSnagDecomp1To3Prob);
      MakeSureIsProportion(m_fSnagDecomp1To4Prob);
      MakeSureIsProportion(m_fSnagDecomp1To5Prob);
      MakeSureIsProportion(m_fSnagDecomp2To2Prob);
      MakeSureIsProportion(m_fSnagDecomp2To3Prob);
      MakeSureIsProportion(m_fSnagDecomp2To4Prob);
      MakeSureIsProportion(m_fSnagDecomp2To5Prob);
      MakeSureIsProportion(m_fSnagDecomp3To3Prob);
      MakeSureIsProportion(m_fSnagDecomp3To4Prob);
      MakeSureIsProportion(m_fSnagDecomp3To5Prob);
      MakeSureIsProportion(m_fSnagDecomp4To4Prob);
      MakeSureIsProportion(m_fSnagDecomp4To5Prob);
      MakeSureIsProportion(m_fSnagDecomp5To5Prob);
      MakeSureGreaterThan(m_fSnagDecompMaxSnagBreakHeight, 0);
      MakeSureGreaterThan(m_fSnagDecompMinSnagBreakHeight, 0);

      if(java.lang.Math.abs(
         m_fSnagDecompLiveTo1Prob.GetValue() +
         m_fSnagDecompLiveTo2Prob.GetValue() +
         m_fSnagDecompLiveTo3Prob.GetValue() +
         m_fSnagDecompLiveTo4Prob.GetValue() +
         m_fSnagDecompLiveTo5Prob.GetValue() - 1) > 0.001 ) {

        throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
           "Probabilities for transitioning from live must add up to 1."));
      }

      if(java.lang.Math.abs(
         m_fSnagDecomp1To1Prob.GetValue() +
         m_fSnagDecomp1To2Prob.GetValue() +
         m_fSnagDecomp1To3Prob.GetValue() +
         m_fSnagDecomp1To4Prob.GetValue() +
         m_fSnagDecomp1To5Prob.GetValue() - 1) > 0.001 ) {

        throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
           "Probabilities for transitioning from class 1 must add up to 1."));
      }

      if(java.lang.Math.abs(
         m_fSnagDecomp2To2Prob.GetValue() +
         m_fSnagDecomp2To3Prob.GetValue() +
         m_fSnagDecomp2To4Prob.GetValue() +
         m_fSnagDecomp2To5Prob.GetValue() - 1) > 0.001 ) {

        throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
           "Probabilities for transitioning from class 2 must add up to 1."));
      }

      if(java.lang.Math.abs(
         m_fSnagDecomp3To3Prob.GetValue() +
         m_fSnagDecomp3To4Prob.GetValue() +
         m_fSnagDecomp3To5Prob.GetValue() - 1) > 0.001 ) {

        throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
           "Probabilities for transitioning from class 3 must add up to 1."));
      }

      if(java.lang.Math.abs(
         m_fSnagDecomp4To4Prob.GetValue() +
         m_fSnagDecomp4To5Prob.GetValue() - 1) > 0.001 ) {

        throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
           "Probabilities for transitioning from class 4 must add up to 1."));
      }

      if(java.lang.Math.abs(m_fSnagDecomp5To5Prob.GetValue() - 1) > 0.001 ) {

        throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
           "Probabilities for transitioning from class 5 must equal 1."));
      }
      
      if (m_fSnagDecompMinSnagBreakHeight.GetValue() >= 
      	 m_fSnagDecompMaxSnagBreakHeight.GetValue()) {
      	throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "The value of \"" + m_fSnagDecompMaxSnagBreakHeight.GetDescriptor() +
        "\" must be greater than the value of \"" + 
        m_fSnagDecompMinSnagBreakHeight.GetDescriptor() + "\"."));
      }
    }
  }
}
